import _extends from "@babel/runtime/helpers/esm/extends";
import _asyncToGenerator from "@babel/runtime/helpers/esm/asyncToGenerator";
import _regeneratorRuntime from "@babel/runtime/regenerator";
import partial from 'lodash.partial';
import { initDomain } from '@resolve-js/core';
import { createQueryExecutor, eventSubscriberFactory } from '@resolve-js/runtime-base';
import { getSecretsManager } from '../../runtime/get-secrets-manager';
import { getEventStore } from '../../runtime/get-event-store';
import { getSagaRuntime } from '../../runtime/get-saga-runtime';
import { mockSideEffects } from '../../runtime/mock-side-effects';
import { getReadModelAdapter } from '../../runtime/get-read-model-adapter';
import { defaultAssertion } from '../../utils/assertions';
import { getCommandImplementationKey, getQueryImplementationKey } from '../../runtime/utils';

var promisedAssertion = function promisedAssertion(assertion, result, error) {
  return new Promise(function (resolve, reject) {
    return assertion(resolve, reject, result, error, false);
  });
};

export var makeTestEnvironment = function makeTestEnvironment(context) {
  var mockedCommandImplementations = new Map();
  var mockedQueryImplementations = new Map();
  var executed = false;
  var useRealSideEffects = false;
  var sideEffectsStartTimestamp = 0;
  var assertions = [];
  var secretsManager = getSecretsManager();
  var completeTest;
  var failTest;

  var setSecretsManager = function setSecretsManager(value) {
    secretsManager = value;
  };

  var allowSideEffects = function allowSideEffects() {
    useRealSideEffects = true;
  };

  var setSideEffectsStartTimestamp = function setSideEffectsStartTimestamp(value) {
    sideEffectsStartTimestamp = value;
  };

  var addAssertion = function addAssertion(value) {
    assertions.push(partial(promisedAssertion, value));
  };

  var isExecuted = function isExecuted() {
    return executed;
  };

  var mockCommandImplementation = function mockCommandImplementation(aggregateName, type, implementation) {
    mockedCommandImplementations.set(getCommandImplementationKey({
      type: type,
      aggregateName: aggregateName
    }), implementation);
  };

  var mockQueryImplementation = function mockQueryImplementation(modelName, resolverName, implementation) {
    mockedQueryImplementations.set(getQueryImplementationKey({
      modelName: modelName,
      resolverName: resolverName
    }), implementation);
  };

  var promise = new Promise(function (resolve, reject) {
    completeTest = resolve;
    failTest = reject;
  });

  var execute = /*#__PURE__*/function () {
    var _ref = _asyncToGenerator( /*#__PURE__*/_regeneratorRuntime.mark(function _callee3() {
      var result, saga, events, adapter, encryption, actualEncryption, actualAdapter, isNext, domain, liveErrors, monitoring, errors, eventSubscriber, executor, eventstoreAdapter, runtime, res, status, liveErrorIndex, _error2, summaryError;

      return _regeneratorRuntime.wrap(function _callee3$(_context3) {
        while (1) {
          switch (_context3.prev = _context3.next) {
            case 0:
              executed = true;
              result = {
                commands: [],
                scheduledCommands: [],
                queries: [],
                sideEffects: []
              };
              saga = context.saga, events = context.events, adapter = context.adapter, encryption = context.encryption;
              actualEncryption = encryption != null ? encryption : /*#__PURE__*/_asyncToGenerator( /*#__PURE__*/_regeneratorRuntime.mark(function _callee() {
                return _regeneratorRuntime.wrap(function _callee$(_context) {
                  while (1) {
                    switch (_context.prev = _context.next) {
                      case 0:
                        return _context.abrupt("return", null);

                      case 1:
                      case "end":
                        return _context.stop();
                    }
                  }
                }, _callee);
              }));

              if (!(adapter != null)) {
                _context3.next = 8;
                break;
              }

              _context3.t0 = adapter;
              _context3.next = 11;
              break;

            case 8:
              _context3.next = 10;
              return getReadModelAdapter();

            case 10:
              _context3.t0 = _context3.sent;

            case 11:
              actualAdapter = _context3.t0;
              isNext = false;
              domain = initDomain({
                viewModels: [],
                readModels: [],
                aggregates: [],
                sagas: [{
                  name: saga.name,
                  handlers: saga.handlers,
                  sideEffects: useRealSideEffects ? saga.sideEffects : mockSideEffects(result, saga.sideEffects),
                  connectorName: 'ADAPTER_NAME',
                  encryption: actualEncryption
                }],
                apiHandlers: []
              });
              liveErrors = [];
              monitoring = {
                group: function group(config) {
                  if (config.Part !== 'ReadModelProjection') {
                    return monitoring;
                  }

                  return _extends({}, monitoring, {
                    error: function (_error) {
                      function error(_x) {
                        return _error.apply(this, arguments);
                      }

                      error.toString = function () {
                        return _error.toString();
                      };

                      return error;
                    }(function (error) {
                      liveErrors.push(error);
                    }),
                    execution: function execution(error) {
                      if (error != null) {
                        liveErrors.push(error);
                      }
                    }
                  });
                },
                time: function time() {
                  return void 0;
                },
                timeEnd: function timeEnd() {
                  return void 0;
                },
                error: function error() {
                  return void 0;
                },
                execution: function execution() {
                  return void 0;
                },
                publish: function () {
                  var _publish = _asyncToGenerator( /*#__PURE__*/_regeneratorRuntime.mark(function _callee2() {
                    return _regeneratorRuntime.wrap(function _callee2$(_context2) {
                      while (1) {
                        switch (_context2.prev = _context2.next) {
                          case 0:
                            return _context2.abrupt("return", void 0);

                          case 1:
                          case "end":
                            return _context2.stop();
                        }
                      }
                    }, _callee2);
                  }));

                  function publish() {
                    return _publish.apply(this, arguments);
                  }

                  return publish;
                }(),
                duration: function duration() {
                  return void 0;
                },
                custom: function custom() {
                  return void 0;
                },
                rate: function rate() {
                  return void 0;
                },
                getMetrics: function getMetrics() {
                  return {
                    metrics: []
                  };
                },
                clearMetrics: function clearMetrics() {
                  return void 0;
                }
              };
              errors = [];
              eventSubscriber = null;
              executor = null;
              _context3.prev = 19;
              _context3.next = 22;
              return getEventStore(events);

            case 22:
              eventstoreAdapter = _context3.sent;
              runtime = getSagaRuntime(result, {
                commands: mockedCommandImplementations,
                queries: mockedQueryImplementations
              }, domain.sagaDomain.schedulerName, secretsManager, monitoring, sideEffectsStartTimestamp);
              eventSubscriber = eventSubscriberFactory({
                getEventSubscriberDestination: function getEventSubscriberDestination() {
                  return 'LOCAL';
                },
                applicationName: 'APP_NAME',
                readModelConnectors: {
                  ADAPTER_NAME: actualAdapter
                },
                getVacantTimeInMillis: function getVacantTimeInMillis() {
                  return 0x7fffffff;
                },
                eventstoreAdapter: eventstoreAdapter,
                readModelsInterop: {},
                performanceTracer: null,
                loadReadModelProcedure: function loadReadModelProcedure() {
                  return Promise.resolve(null);
                },
                setCurrentEventSubscriber: function setCurrentEventSubscriber() {
                  return void 0;
                },
                sagasInterop: domain.sagaDomain.acquireSagasInterop(runtime),
                monitoring: null
              });
              executor = createQueryExecutor({
                readModelConnectors: {
                  ADAPTER_NAME: actualAdapter
                },
                readModelsInterop: domain.sagaDomain.acquireSagasInterop(runtime),
                viewModelsInterop: {},
                performanceTracer: null,
                monitoring: null,
                eventSubscriber: eventSubscriber
              });
              _context3.prev = 26;
              _context3.next = 29;
              return executor.subscribe({
                modelName: saga.name,
                subscriptionOptions: {
                  eventTypes: null,
                  aggregateIds: null
                }
              });

            case 29:
              _context3.next = 31;
              return executor.resume({
                modelName: saga.name
              });

            case 31:
              isNext = false; // TODO ???

              _context3.next = 34;
              return executor.build({
                modelName: saga.name
              });

            case 34:
              res = _context3.sent;

              if (res.payload["continue"]) {
                isNext = true;
              }

            case 36:
              if (isNext) {
                _context3.next = 31;
                break;
              }

            case 37:
              _context3.next = 39;
              return executor.status({
                modelName: saga.name
              });

            case 39:
              status = _context3.sent;

              if (!(status.status === 'error')) {
                _context3.next = 51;
                break;
              }

              if (Array.isArray(status.errors)) {
                _context3.next = 43;
                break;
              }

              throw new Error('Unknown error');

            case 43:
              liveErrorIndex = status.errors.length === 1 ? liveErrors.findIndex(function (err) {
                return err.message === status.errors[0].message;
              }) : -1;

              if (!(liveErrorIndex < 0)) {
                _context3.next = 50;
                break;
              }

              _error2 = new Error(status.errors.map(function (err) {
                return err.message;
              }).join('\n'));
              _error2.stack = status.errors.map(function (err) {
                return err.stack;
              }).join('\n');
              throw _error2;

            case 50:
              throw liveErrors[liveErrorIndex];

            case 51:
              _context3.next = 56;
              break;

            case 53:
              _context3.prev = 53;
              _context3.t1 = _context3["catch"](26);
              errors.push(_context3.t1);

            case 56:
              _context3.prev = 56;
              _context3.next = 59;
              return executor.unsubscribe({
                modelName: saga.name
              });

            case 59:
              _context3.next = 64;
              break;

            case 61:
              _context3.prev = 61;
              _context3.t2 = _context3["catch"](56);
              errors.push(_context3.t2);

            case 64:
              _context3.next = 69;
              break;

            case 66:
              _context3.prev = 66;
              _context3.t3 = _context3["catch"](19);
              errors.push(_context3.t3);

            case 69:
              if (!(errors.length === 0)) {
                _context3.next = 85;
                break;
              }

              if (!(assertions.length > 0)) {
                _context3.next = 82;
                break;
              }

              _context3.prev = 71;
              _context3.next = 74;
              return Promise.all(assertions.map(function (assertion) {
                return assertion(result, null);
              }));

            case 74:
              completeTest(result);
              _context3.next = 80;
              break;

            case 77:
              _context3.prev = 77;
              _context3.t4 = _context3["catch"](71);
              failTest(_context3.t4);

            case 80:
              _context3.next = 83;
              break;

            case 82:
              defaultAssertion(completeTest, failTest, result, null);

            case 83:
              _context3.next = 101;
              break;

            case 85:
              summaryError = errors[0];

              if (errors.length > 1) {
                summaryError = new Error(errors.map(function (err) {
                  return err.message;
                }).join('\n'));
                summaryError.stack = errors.map(function (err) {
                  return err.stack;
                }).join('\n');
              } // eslint-disable-next-line no-console


              console.error(summaryError);

              if (!(assertions.length > 0)) {
                _context3.next = 100;
                break;
              }

              _context3.prev = 89;
              _context3.next = 92;
              return Promise.all(assertions.map(function (assertion) {
                return assertion(result, errors[0]);
              }));

            case 92:
              failTest(errors[0]);
              _context3.next = 98;
              break;

            case 95:
              _context3.prev = 95;
              _context3.t5 = _context3["catch"](89);
              failTest(_context3.t5);

            case 98:
              _context3.next = 101;
              break;

            case 100:
              defaultAssertion(completeTest, failTest, result, errors[0]);

            case 101:
            case "end":
              return _context3.stop();
          }
        }
      }, _callee3, null, [[19, 66], [26, 53], [56, 61], [71, 77], [89, 95]]);
    }));

    return function execute() {
      return _ref.apply(this, arguments);
    };
  }();

  setImmediate(execute);
  return {
    setSecretsManager: setSecretsManager,
    allowSideEffects: allowSideEffects,
    setSideEffectsStartTimestamp: setSideEffectsStartTimestamp,
    addAssertion: addAssertion,
    isExecuted: isExecuted,
    promise: promise,
    mockCommandImplementation: mockCommandImplementation,
    mockQueryImplementation: mockQueryImplementation
  };
};
//# sourceMappingURL=make-test-environment.js.map